
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <jni.h>

#include "SerialPort.h"

#include "android/log.h"

#ifdef DEPENDING_GPIO
#include "GPIOControl.h"
#endif

static const char *TAG = "serial_port";
#define LOGI(fmt, args...) __android_log_print(ANDROID_LOG_INFO,  TAG, fmt, ##args)
#define LOGD(fmt, args...) __android_log_print(ANDROID_LOG_DEBUG, TAG, fmt, ##args)
#define LOGE(fmt, args...) __android_log_print(ANDROID_LOG_ERROR, TAG, fmt, ##args)

/*
static speed_t getBaudrate(int baudrate)
{
	switch(baudrate) {
		case 0: return B0;
		case 50: return B50;
		case 75: return B75;
		case 110: return B110;
		case 134: return B134;
		case 150: return B150;
		case 200: return B200;
		case 300: return B300;
		case 600: return B600;
		case 1200: return B1200;
		case 1800: return B1800;
		case 2400: return B2400;
		case 4800: return B4800;
		case 9600: return B9600;
		case 19200: return B19200;
		case 38400: return B38400;
		case 57600: return B57600;
		case 115200: return B115200;
		case 230400: return B230400;
		case 460800: return B460800;
		case 500000: return B500000;
		case 576000: return B576000;
		case 921600: return B921600;
		case 1000000: return B1000000;
		case 1152000: return B1152000;
		case 1500000: return B1500000;
		case 2000000: return B2000000;
		case 2500000: return B2500000;
		case 3000000: return B3000000;
		case 3500000: return B3500000;
		case 4000000: return B4000000;
		default: return -1;
	}
}
*/

CSerialPort::CSerialPort() : fd(-1)
{
}

CSerialPort::~CSerialPort()
{
	if (fd != -1)
	{
		close(fd);
	}
}

bool CSerialPort::Open(const char* path, speed_t baudrate, int dataBits, int stopBits, int parity, int vtime, int vmin)
{
	if (baudrate == -1) {
		return false;
	}

	/* Opening device */
	{
		fd = open(path, O_RDWR | flags);
		if (fd == -1) {
			return false;
		}
	}

	/* Configure device */
	if (SetAttributes(baudrate, dataBits, stopBits, parity, vtime, vmin))
	{
		LOGE("tcsetattr() failed");
		close(fd);
		/* TODO: throw an exception */
		return false;
	}

	return true;
}

void CSerialPort::Close()
{
	if (fd > 0)
	{
		close(fd);
		fd = -1;
	}
}

int CSerialPort::SetAttributes(unsigned int baudrate, int databit, int stopbit, char parity, int vtime, int vmin)
{
	struct termios opt;
	tcgetattr(fd, &opt);
	set_baudrate(&opt, baudrate);
	//opt.c_cflag |= CLOCAL|CREAD; /*|CRTSCTS */
	opt.c_lflag &= ~(ICANON | ECHO |ECHOE |ISIG);
	set_data_bit(&opt, databit);
	set_parity(&opt, parity);
	set_stopbit(&opt, stopbit);
	opt.c_iflag &=~(INLCR|ICRNL);
	opt.c_iflag &=~(IXON);/* 流控*/
	opt.c_oflag = 0;
	//opt.c_lflag |= 0;
	opt.c_oflag &= ~OPOST;
	opt.c_cc[VTIME] = vtime;
	opt.c_cc[VMIN] = vmin;
	tcflush (fd, TCIFLUSH);
	return (tcsetattr (fd, TCSANOW, &opt));
}

size_t CSerialPort::Send(const uint8_t* data, size_t dataLength)
{
	if (fd < 0)
	{
		return -1;
	}
	size_t bytesWritten = write(fd, data, dataLength);
	return bytesWritten;
}

size_t CSerialPort::Recv(uint8_t* buffer, size_t bufferLength)
{
	if (fd < 0)
	{
		return -1;
	}

	size_t bytesRead = read(fd, buffer, bufferLength);/* 在串口读取字符串 */

	return bytesRead;
#if 0
	// sprintf(buf, "收到串口%d, %d字节数据:", j + 1, recvlen);
	// BytestreamLOG(j, buf, recvbuf, recvlen, 'I');
#endif
	// SerialDataProcess(srdt.curdevidx[j], recvbuf, recvlen);
}